      INTEGER FUNCTION CRMPSC(sec_name,chan,inadr,retadr)
C
C-----------------------------------------
C
C   Author   :- J.Huber               14-Apr-1991
C
C=========================================
C
C   Purpose   :
C <title>  create and map a global section file </title>
C   Returned value  : system service $CRMPSC completion code
C   Inputs    :
C       character*(*) sec_name:   name of the global section
C       integer chan      :    channel to section file
C       integer inadr(2)  :    address range to map (begin-,end-address)
C   Outputs   :
C       integer retadr(2)  :    address range actually mapped
C
C   notes:

C1)     the section file must have been connected already to have a valid 
C       channel number. Use GBL_SECTION_UFO for that purpose.
C2)     the address range is a pair of addresses. Get it in Fortran e.g. by
C       Byte front_guard(512),rear_guard(512)
C       Integer data(2048)
C       Common /global/front_guard,data,rear_guard
C       inadr(1)=%LOC(front_guard(512))
C       inadr(2)=%LOC(rear_guard(1))
C	Global-sections are always mapped in units of pages: from BEGIN of the
C	page inadr(1) up to the END of the page containing inadr(2).
C       See VMS system-services ref. manaul for details
C3)     Retadr can be used as a check, if the entire data section is mapped.
C4)     Entry at CRMPSC_RO maps the section with Read-Only access
C Addendum for VMS on AXP:
C5)     On Alpha, the addresses must be page aligned, use appropriate linker
C       options (PSECT_ATTR = gblsec , PAGE,SHR,OVR), or guard-pages.
C       Compile common sections in DECC with 
C              /EXTERN_MODEL=COMMON_BLOCK/SHARE_GLOBALS

C start-address and end-address must be on a page
C boundary, i.e. also the length must be a multiple of the systems page size.
C the calling program must have guard-pages <pagesize> long.
C There is a system-service and a Crtl function getpagesize to make this 
C architecture dependant. Currently pagesize for Vax is 512, for AXP 8192.
C To make this somewhat automatic, I have an include file vms_pagesize.inc in
C each systems sys$library like this:
C Vax:
* file vms_pagesize.inc for VAX processors
*      Integer VMS_PAGESIZE                !page-size in bytes
*      Integer VMS_PAGESIZE_I4             !page-size in Integers
*      Parameter (VMS_PAGESIZE = 512)   !VAX
*      Parameter (VMS_PAGESIZE_I4 = 128)   !VAX
C AXP:
* file vms_pagesize.inc for Alpha processors
*      Integer VMS_PAGESIZE                !page-size in bytes
*      Integer VMS_PAGESIZE_I4             !page-size in Integers
*      Parameter (VMS_PAGESIZE_I4 = 2048)   !Alpha integer*4
*      Parameter (VMS_PAGESIZE = 8192)     !Alpha (3000/400) page size bytes

C Then define integer front_guard(VMS_PAGESIZE_I4),
C             integer rear_guard(VMS_PAGESIZE_I4)
C and at run-time, verify the result of getpagesize() equals VMS_PAGESIZE:
*      pagesize = getpagesize()
*      if (pagesize.ne.VMS_PAGESIZE) stop ' VMS pagesize mismatch'
*      inadr(1) = %LOC(front_guard(1))      !virtual start-address 
*      inadr(2) = IOR(%LOC(rear_guard(1)),pagesize-1) !align length of section
*      pagcnt = (inadr(2)-inadr(1))/512   !number of VMS pagelets

C=========================================
C +
C Declarations.
C -
      IMPLICIT NONE
      INTEGER CHAN,INADR(2),RETADR(2)
      CHARACTER*(*) sec_name
      INTEGER CRMPSC_RO,MGBLSC,MGBLSC_RO

      INCLUDE '($SECDEF)'
      INCLUDE '($SSDEF)'
* Mask Values for CRMPSC system service
      INTEGER Mask
* System services and RTL routines
      INTEGER SYS$CRMPSC,SYS$MGBLSC
*       Structure for ident field of CRMPSC system service
      STRUCTURE /Match/
        INTEGER*4 criterion
        UNION
          MAP
            INTEGER*4 Minor_ID
          END MAP
          MAP
            BYTE Major_ID(4)
          END MAP
        END UNION
      END STRUCTURE
      RECORD /Match/ ident


C + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
C Main Entry Point.
C - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

*     create global section and map
      mask = SEC$M_WRT .OR. SEC$M_GBL
  100 CONTINUE   !common part

      Ident.criterion = SEC$K_MATALL
      Ident.Minor_ID = 1
      Ident.Major_ID(4) = 1

      CRMPSC = SYS$CRMPSC(INADR,RETADR,,%VAL(Mask),sec_name,
     &          Ident,,%VAL(chan),,,,)
      RETURN
C + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
*     Read-Only mapping entry
C + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
      ENTRY CRMPSC_RO(sec_name,chan,inadr,retadr)
      mask =  SEC$M_GBL
      GOTO 100  !join common part
C + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
*     Map an already existing section entry
*        (the chan argument is an unused placeholder for this entry).
C + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
      ENTRY MGBLSC(sec_name,chan,inadr,retadr)
      mask = SEC$M_WRT .OR. SEC$M_GBL
      GOTO 200
      ENTRY MGBLSC_RO(sec_name,chan,inadr,retadr)
      mask =  SEC$M_GBL
 200  CONTINUE
      Ident.criterion = SEC$K_MATALL
      Ident.Minor_ID = 1
      Ident.Major_ID(4) = 1
      MGBLSC_RO = SYS$MGBLSC(INADR,RETADR,,%VAL(Mask),sec_name,
     &          Ident,,)
      END
