/* 
   long cvtnum ( char *string);
<META name="keywords" content="convert,string,number">   
<title>     Convenience routine to convert a string to a number. </title>   
   
   string can be a decimal,octal, or hexdecimal number.
   hexadecimal may be noted as "0xhexdigits", or are recognized automatically
   by occurances of digits A-F.
   Numbers starting with leading zeroes without a following x are 
   interpreted octal.
   Hex and Octal also can be denoted as %X,%O (VMS DCL compatibility).
   
   Syntax errors (characters other than digits) return value 0 and errno = -1;
   No error will be detected if "x" or "O" follows some legal number: the preceeding
   number will be hex/octal-converted.
*/

#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <stdio.h>
#include <errno.h>

 long cvtnum ( char * string ) {
  int c=0,prev=0;
  int i,radix = 10, start = 0;
  char *begin,*end;
  
  errno = 0;
  for ( i=0; i < strlen(string); i++) {
      prev = c;
      c=toupper( string[i] );
      if ( isspace(c) ) { start = (i+1); continue; } /* whitespace */
      if (c == '%') {start = (i+1); continue;}
      if ( (i == start) && (c == '0') ) {radix = 8; continue ;} /* octal */
      if ( (i == start) && ( (c == '-') || (c == '+'))) continue ; /* sign */
      if ( isdigit(c) ) continue;
      if ( isxdigit(c) ) { radix = 16; continue; }
      if ( (i > 0) && (c == 'X') ) 
        { radix = 16; continue; }  /* numX hex */
      if ( (i > 0) && (c == 'O') ) 
        { radix = 8; continue; }  /* numO octal */
      if ( (i > 0) && (prev == '%') && (c == 'X') ) 
        { radix = 16; start = (i+1); continue; }  /* %Xhex */
      if ( (i > 0) && ((prev == '0')||(prev == '%')) && (c == 'O') ) 
        { radix = 8; start = (i+1); continue; }  /* 0Ooct,%Ooct */
#ifdef DEBUG
printf("cvtnum: Illegal character in input: %c\n",c);
#endif
      errno = -1;
      return 0;
     }
   begin = string;
  if ( start ) begin += start;
#ifdef DEBUG
printf("cvtnum: start is %d string at begin: %s\n",start,begin);
#endif
  return ( strtoul( begin, &end, radix ) );
  }
#ifdef TEST
main () {

  long x;
  char * end;
  char line[80];

  while (gets(line)) {
  x = cvtnum (line);
  printf("String: %s : result = 0x%x , %d - errno %d\n",line,x,x,errno);
   }
}
#endif
