
/*<TITLE>HG$GET_INPUT provide command recall when reading from SYS$INPUT</TITLE>
 This is a C version of the original Vax Macro routine written by Hunter Goatley.
 C coding by Joseph Huber.
;++
;
;  Routine:	HG$GET_INPUT
;
;  Author:	Hunter Goatley, Western Kentucky University
;		goathunter@WKUVX1.BITNET
;
;  Date:	January 18, 1988
;
;  Functional Description:
;
;	This routine implements an easy-to-call interface to the SMG$ routines
;	to provide command recall when reading from SYS$INPUT.	It takes
;	the same arguments as LIB$GET_INPUT and may be substituted directly
;	in any call to LIB$GET_INPUT.
;
;	Example:
;
;		CALL LIB$GET_INPUT (buffer, prompt, lbuffer)
;			to
;		CALL HG$GET_INPUT (buffer, prompt, lbuffer)
;
;
;	HG$GET_INPUT  get-str [,prompt-str] [,out-len]
;
;  Inputs:
;
;	get-str	- String which is read from SYS$INPUT.
;		  Address of string descriptor.   Write-only.
;	prompt	- Prompt message that is displayed.
;		  Address of string descriptor.  Read-only.
;	out-len	- Number of bytes written into get-str.
;		  Address of word.  Write-only.
;
;  Returns:
;
;	String in get-str and status in return code.
;
;  Effects:
;
;	On first call, a key table and a virtual keyboard are created.
;
;-
*/

#include string
#include descrip
#include rmsdef			/*; RMS symbols */
#include ssdef			/*; System service status symbols */
#include smgdef			/*; SMG$ symbols */
#include smg$routines
#include stdarg
 
static long KEY_TABLE_ID;	/*; ID of key table created*/
static long KEYBOARD_ID;	/*; ID of virtual keyboard created*/
 

int HG$GET_INPUT(struct dsc$descriptor * buf, ... ) 
{

  short * lbuf=NULL;
  struct dsc$descriptor * prompt=NULL;
  int		nargs;
  int st=0;
  va_list va;
/*
 * Pick up the arguments, forcing all missing ones to NULL
 */
  va_start(va,buf);
  va_count(nargs);
  if (nargs > 1) prompt = va_arg(va,struct dsc$descriptor *);
  if (nargs > 2) lbuf = va_arg(va,short *);
  va_end(va);

  if ( KEY_TABLE_ID == 0) { 
    st = smg$create_key_table(&KEY_TABLE_ID);
    if (st&1 != 1) return st ;
    st = smg$create_virtual_keyboard(&KEYBOARD_ID);
    if (st&1 != 1) return st ;
  } 
  
  st = smg$read_composed_line(&KEYBOARD_ID,&KEY_TABLE_ID,buf,prompt,lbuf);
        /* Was ^Z entered? Make status RMS$_EOF */
	if (st == SMG$_EOF) st=RMS$_EOF; 
  return st; 
}
