!<TITLE>HG$GET_INPUT provide command recall when reading from SYS$INPUT</TITLE>
! This is a Fortran version of the original Vax Macro routine written
! by Hunter Goatley.
! Coding by Joseph Huber.
!++
!  Routine:     HG$GET_INPUT
!  Author:      Hunter Goatley, Western Kentucky University
!               goathunter@WKUVX1.BITNET
!  Date:        January 18, 1988
!  Functional Description:
!
!       This routine implements an easy-to-call interface to the SMG$ routines
!       to provide command recall when reading from SYS$INPUT.  It takes
!       the same arguments as LIB$GET_INPUT and may be substituted directly
!       in any call to LIB$GET_INPUT.
!
!       Example:
!               CALL LIB$GET_INPUT (buffer, prompt, lbuffer)
!                       to
!               CALL HG$GET_INPUT (buffer, prompt, lbuffer)
!
!       HG$GET_INPUT  get-str [,prompt-str] [,out-len]
!  Inputs:
!       get-str - String which is read from SYS$INPUT.
!                 Address of string descriptor.   Write-only.
!       prompt  - Prompt message that is displayed.
!                 Address of string descriptor.  Read-only.
!       out-len - Number of bytes written into get-str.
!                 Address of word.  Write-only.
!  Returns:
!       String in get-str and status in return code.
!  Effects:
!       On first call, a key table and a virtual keyboard are created.
!-
      INTEGER FUNCTION HG$GET_INPUT(buf, prompt, lbuf )
      IMPLICIT NONE
      CHARACTER*(*) buf
      BYTE prompt(*)  !cheating about type: may be missing
      INTEGER*2 lbuf

      INCLUDE '($rmsdef)'                     ! RMS symbols */
      INCLUDE '($ssdef)'                      ! System service status symbols */
      INCLUDE '($smgdef)'                     ! SMG$ symbols */
      INCLUDE '($smgmsg)'                     ! SMG$ symbols */
      INCLUDE '(smg$routines)'

      INTEGER KEY_TABLE_ID    !! ID of key table created*/
      INTEGER KEYBOARD_ID     !! ID of virtual keyboard created*/

      DATA KEY_TABLE_ID /0/
      DATA KEYBOARD_ID /0/
      SAVE KEY_TABLE_ID
      SAVE KEYBOARD_ID

      hg$get_input = 1
      IF ( KEY_TABLE_ID .EQ. 0) THEN
        hg$get_input = smg$create_key_table(KEY_TABLE_ID)
        IF (hg$get_input) hg$get_input =
     &    smg$create_virtual_keyboard(KEYBOARD_ID)
      ENDIF
      IF (.NOT.hg$get_input) RETURN

      IF (iargcount().LT.2) THEN
        hg$get_input = smg$read_composed_line(KEYBOARD_ID,
     &     KEY_TABLE_ID,buf)
      ELSE IF (iargcount().LT.3) THEN
        hg$get_input = smg$read_composed_line(KEYBOARD_ID,
     &     KEY_TABLE_ID,buf,prompt)
      ELSE
        hg$get_input = smg$read_composed_line(KEYBOARD_ID,
     &     KEY_TABLE_ID,buf,prompt,lbuf)
      ENDIF

        ! Was ^Z entered? Make status RMS$_EOF */
      IF (hg$get_input .EQ. SMG$_EOF) hg$get_input=RMS$_EOF!
      END
