/*<title>Create a permanent mailbox</title>
 prmmbx
 *   Create a permanent mailbox via the $CREMBX system service.
 *   Requires PRMMBX privilege to work.
 *
 *     Args:    argv[1] - maxmsg - maximum size in bytes of a message
 *                        that can be sent to this mailbox
 *              argv[2] - bufquo - how big this mbx's buffer will be
 *              argv[3] - prclognam - Process logical name to point to mbx.
 *                        Defaults to `UA_PRMMBX'.
 *
 * Last modified:  30-Aug-1991/jsh
 *
 * 30-Aug-1991  jsh     remove LNM stuff: log name in lnm$system
 *  2-Jan-1990  al      Do LNM creation right
 * 29-Dec-1989  al      First version
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include climsgdef
#include descrip
#include lnmdef
#include psldef
#include stsdef
#include <str$routines.h>
#define VMSERR( s ) if (!($VMS_STATUS_SUCCESS(s))) exit(s)   
#define DEFPRCLOGNAM "UA_PRMMBX"                /* default LNM for new MBX */
#define PERMANENT 1
#define TEMPORARY 0

extern int main(int argc, char **argv)
{
    unsigned long int status, SYS$CREMBX();
    int atoi();
    size_t strlen();
    void gripeaboutusage();

    unsigned short int chan;
    unsigned long int maxmsg, bufquo;
    char *defprclognam = DEFPRCLOGNAM;
    struct dsc$descriptor prclognamD;   

    /* First, sanity-check the args.  argv[1] (maxmsg) defaults to 0;
     * argv[2] (bufquo) defaults to 0; argv[3] (prclognam) defaults to
     * "UA_PRMMBX".
     */

    if (argc > 4)
    {
        gripeaboutusage();
        exit(CLI$_MAXPARM);
    }

    if (argc < 2)
        maxmsg = 0;
    else
        sscanf(argv[1], "%d", &maxmsg);

    if (argc < 3)
        bufquo = 0;
    else
        sscanf(argv[2], "%d", &bufquo);

    if (argc == 4)
    {
        prclognamD.dsc$a_pointer = argv[3];
        prclognamD.dsc$w_length = (unsigned short int) strlen(argv[3]);
    }
    else
    {
        prclognamD.dsc$a_pointer = defprclognam; 
        prclognamD.dsc$w_length = strlen(defprclognam);
    }
    prclognamD.dsc$b_dtype = DSC$K_DTYPE_T;
    prclognamD.dsc$b_class = DSC$K_CLASS_S;

/* make the logical name upper case,
   it's annoying not to find it with show log*/
    str$upcase(&prclognamD,&prclognamD);
    /* Now call $CREMBX, specifying the following args:
     * prmflg <- 1 (permanent)
     * chan
     * maxmsg
     * bufquo
     * promsk <- 0 (all access allowed; user should SET DEV/PROT after calling)
     * acmode <- PSL$C_USER
     * lognam <- prclognamD
     */

    status = SYS$CREMBX(PERMANENT, &chan, maxmsg, bufquo, 0, PSL$C_USER, 
                        &prclognamD);
    VMSERR(status);
    exit(status); 

}   /* main */

/* gripeaboutusage
 *   complain to SYS$ERROR
 */
void gripeaboutusage()
{
/*    int fprintf();  */
    fprintf(stderr, "Usage: prmmbx [maxmsg] [bufquo] [lognam]\n");
    fprintf(stderr, 
"where: maxmsg is the maximum size in bytes of a message that can be sent\n");
    fprintf(stderr, "       to the mailbox\n");
    fprintf(stderr, "       bufquo is buffer size for mbx (max: 64k bytes)\n");
    fprintf(stderr, 
"       lognam is the system LNM that will point to the mbx.\n");
}
