/* <title>Create a permanent mailbox</title>
 *   Create a permanent mailbox via the $CREMBX system service.
 *   Requires PRMMBX privilege to work.
 *
 *     Args:    argv[1] - maxmsg - maximum size in bytes of a message
 *                        that can be sent to this mailbox
 *              argv[2] - bufquo - how big this mbx's buffer will be
 *              argv[3] - prclognam - Process logical name to point to mbx.
 *                        Defaults to `UA_PRMMBX'.
 *
 * Last modified:  2-Jan-1990/al
 *
 *  2-Jan-1990  al      Do LNM creation right
 * 29-Dec-1989  al      First version
 *
 */

#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include climsgdef
#include descrip
#include lnmdef
#include psldef
#include stsdef
#include <lib$routines.h>
#include <starlet.h>

#define VMSERR( s ) if (!($VMS_STATUS_SUCCESS(s))) exit(s)   
#define DEFPRCLOGNAM "UA_PRMMBX"                /* default LNM for new MBX */
#define PERMANENT 1
#define TEMPORARY 0

typedef struct _item {
    unsigned short	length;			/* length of data buffer */
    unsigned short	code;			/* item code */
    char	        *buffer;		/* data buffer */
    int	                *retlen;		/* returned item length */
} ITEM;

extern int main(int argc, char **argv)
{
    unsigned long int status;
    int atoi();
    size_t strlen();
    void gripeaboutusage();

    unsigned short int chan;
    unsigned long int maxmsg, bufquo;
    char *defprclognam = DEFPRCLOGNAM;
    $DESCRIPTOR(lnmtableD, "LNM$PROCESS_DIRECTORY");
    $DESCRIPTOR(lnmD, "LNM$PERMANENT_MAILBOX");
    $DESCRIPTOR(tmplnmD, "UA_TMP_PRMMBX");
    $DESCRIPTOR(prctabD, "LNM$PROCESS_TABLE");
    char *lnmval = "LNM$PROCESS_TABLE";
    struct dsc$descriptor prclognamD;   
    ITEM itmlst[2];             /* itemlist for $CRELNM call */
    char mbxname[LNM$C_NAMLENGTH+1];
    int mbxnamelength;
    struct dsc$descriptor mbxnameD;

    /* First, sanity-check the args.  argv[1] (maxmsg) defaults to 0;
     * argv[2] (bufquo) defaults to 0; argv[3] (prclognam) defaults to
     * "UA_PRMMBX".
     */

    if (argc > 4)
    {
        gripeaboutusage();
        exit(CLI$_MAXPARM);
    }

    if (argc < 2)
        maxmsg = 0;
    else
        sscanf(argv[1], "%d", &maxmsg);

    if (argc < 3)
        bufquo = 0;
    else
        sscanf(argv[2], "%d", &bufquo);

    if (argc == 4)
    {
        prclognamD.dsc$a_pointer = argv[3];
        prclognamD.dsc$w_length = (unsigned short int) strlen(argv[3]);
    }
    else
    {
        prclognamD.dsc$a_pointer = defprclognam; 
        prclognamD.dsc$w_length = strlen(defprclognam);
    }
    prclognamD.dsc$b_dtype = DSC$K_DTYPE_T;
    prclognamD.dsc$b_class = DSC$K_CLASS_S;

    /* Define the LNM LNM$PERMANENT_MAILBOX in user mode to go into
     * the LNM$PROCESS_TABLE LNM table.  This is to make the LNM pointing
     * to the newly created MBX go into the job table rather than into
     * the system table which is the default for permanent MBXes.  The
     * LNM$PERMANENT_MAILBOX LNM is created in user mode so as to vanish
     * when this image runs down.
     */

    itmlst[0].length = (unsigned short int) strlen(lnmval);
    itmlst[0].code = LNM$_STRING;
    itmlst[0].buffer = lnmval;                
    itmlst[1].length = itmlst[1].code = 0;

    status = sys$crelnm(0, &lnmtableD, &lnmD, &PSL$C_USER, &itmlst);
    VMSERR(status);

    /* Now call $CREMBX, specifying the following args:
     * prmflg <- 1 (permanent)
     * chan
     * maxmsg
     * bufquo
     * promsk <- 0 (all access allowed; user should SET DEV/PROT after calling)
     * acmode <- PSL$C_USER
     * lognam <- tmplnmD
     */

    status = sys$crembx(PERMANENT, &chan, maxmsg, bufquo, 0, PSL$C_USER, 
                        &tmplnmD);
    VMSERR(status);

    /* The identity of the mailbox is now contained in the user mode process
     * LNM pointed to by tmplnmD.  As this LNM will vanish when this image
     * runs down, we must put its value into a super mode LNM.
     */          

    itmlst[0].length = LNM$C_NAMLENGTH;
    itmlst[0].code = LNM$_STRING;
    itmlst[0].buffer = mbxname;
    itmlst[0].retlen = &mbxnamelength;
    status = sys$trnlnm(0, &prctabD, &tmplnmD, &PSL$C_USER, &itmlst);   
    VMSERR(status);

    mbxnameD.dsc$w_length = (unsigned short int) mbxnamelength;
    mbxnameD.dsc$a_pointer = mbxname;
    mbxnameD.dsc$b_dtype = DSC$K_DTYPE_T;
    mbxnameD.dsc$b_class = DSC$K_CLASS_S;
    status = lib$set_logical(&prclognamD, &mbxnameD, 0, 0, 0);
    exit(status); 

}   /* main */

/* gripeaboutusage
 *   complain to SYS$ERROR
 */
void gripeaboutusage()
{
    fprintf(stderr, "Usage: prmmbx [maxmsg] [bufquo] [prclognam]\n");
    fprintf(stderr, 
"where: maxmsg is the maximum size in bytes of a message that can be sent\n");
    fprintf(stderr, "       to the mailbox\n");
    fprintf(stderr, "       bufquo is buffer size for mbx (max: 64k bytes)\n");
    fprintf(stderr, 
"       prclognam is the process LNM that will point to the mbx.\n");
}
