        OPTIONS /EXTEND_SOURCE
        PROGRAM TrimCard
*++
*       TrimCard.FOR
* Created: 31-jul-1990 J.Huber
*<title>Trim records in text files to max. 80 bytes, no trailing blanks</title>
* TrimCard reads a (cardimage-) input-file and writes a new output file
* with the same content, but
* o all text beyond column 80 removed, and
* o all trailing blanks removed.
* The size of cardimage files coming from fixed length record datasets
* (e.g. exported from IBM systems) is thus substantially reduced.
* This program originally was a quick hack at transition from IBM VM cardimage
* files to VMS and Unix, but now enhanced.
* There is a VMS freeware TRIM, which has more options
* like adding prefix/suffix, but lacks the /COLUMN and /WRAP options.
*
** usage:
* Trimcard [/COLUMN:n][/WRAP] infile [outfile]
* Examples:
*        TrimCard <filespec> 
* will produce a new version of the file <filespec>.
*        TrimCard <infile>  <outfile>
* puts the trimmed records in outfile.
*
* Qualifiers: 
*  /COLUMN:n   cuts at specified number of characters
*  /WRAP  does not cut the records at /columns, but wraps to new lines
*   every /column:n characters.
*
* Libraries needed:
*  UTIL/LIB: init_cli 

C        Copyright (C) 1990   Joseph Huber (huber@mppmu.mpg.de)

C        This program is free software; you can redistribute it and/or modify
C        it under the terms of the GNU General Public License as published by
C        the Free Software Foundation; either version 2 of the License, or
C        (at your option) any later version.

C        This program is distributed in the hope that it will be useful,
C        but WITHOUT ANY WARRANTY; without even the implied warranty of
C        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C        GNU General Public License for more details.
*--
        IMPLICIT NONE
        CHARACTER*4096 cardin
        CHARACTER*4096 cardout
        CHARACTER*254 file,file2
        INTEGER*2 lout
	integer column,status,lwrap,idelim,iostat
        logical wrap,words

        integer delimit
        external delimit
        integer iostat_to_status
        external iostat_to_status
        external TRIM_TABLE  !CLI syntax table from TRIMCARD_TABLE.CLD

        Include '($SYSSRVNAM)'
        INCLUDE '($CLIDEF)'
        INCLUDE '(CLI$routines)'
        INCLUDE '(LIB$ROUTINES)'
        INCLUDE '(OTS$ROUTINES)'
        INCLUDE '($LIBDEF)'
      integer CLI$_NEGATED
      external CLI$_NEGATED

! TRIM_TABLE points to a CLI table defined like:
!Module TRIM_TABLE
!Define Verb TRIM
!Parameter P1
! Value (Type=$FILE, required)
!Qualifier HELP
!Qualifier COLUMN (Type=$number, default:80) !    trim to length columns
!Qualifier WRAP   ! wrap at column, do not cut
!qualifier WORD   ! warp, but don't cut inside words

	column=80
        wrap=.false.
        words=.false.

        call init_cli_for(TRIM_TABLE,'TRIMCARD')
        if (cli$present('HELP')) then
        Print *," Trimcard [/COLUMN:n][/WRAP/WORD] infile [outfile]"
        Print *,' TRIMCARD reads a (cardimage-) input-file and writes a'
	Print *,' new output file with the same content, but '
	Print *,' o all text beyond column /COLUMN:n [80] removed, and'
	Print *,' o all trailing blanks removed.'
        Print *,' /WRAP : do not cut at column, but wrap to next line.'
        Print *,' /WORD : wrap, but at word boundaries.'
        Call Exit(1)
        endif
 
        status = cli$present('WRAP')
        IF (status.and.(status.ne.%LOC(CLI$_NEGATED))) wrap = .TRUE.
        status = cli$present('WORD')
        IF (status.and.(status.ne.%LOC(CLI$_NEGATED))) THEN
          wrap = .TRUE.
          words=.TRUE.
        ENDIF
        if (cli$present('COLUMN')) then
	 status=cli$get_value('COLUMN',file,lout)
	 if (status) then
	  status=OTS$CVT_TI_L(file(1:lout),column,%VAL(4),%VAL(1))
        endif
        endif
        if (cli$present('P1')) then
	  status=cli$get_value('P1',file,lout)
          file2=file
          OPEN(unit=1,name=file,type='OLD',readonly,err=899,iostat=iostat)
          if (cli$present('P2')) status=cli$get_value('P2',file2,lout)
          OPEN(unit=2,name=file2,type='NEW',carriagecontrol='LIST',
     &          err=898,iostat=iostat)

          DO WHILE (.true.)
            READ(1,1000,end=999) cardin
            CALL str$trim(cardin,cardin,lout)
            lout=MAX(lout,1)
            DO WHILE (lout.gt.0)
             lwrap=column
             if (wrap.AND.words) then
               idelim=delimit(cardin(1:lwrap))
               if (idelim.gt.1) lwrap=idelim
             endif        
             cardout = cardin(1:MIN(lwrap,lout))
             CALL str$trim(cardout,cardout,lout)
             IF (lout.LE.0) lout=1
             WRITE(2,2000) cardout(1:lout)
D       Print *,' lout:',lout
             lout=0
             IF (wrap) THEN
              cardin = cardin(lwrap+1:)
              CALL str$trim(cardin,cardin,lout)
             ENDIF
            ENDDO
          ENDDO

  999     CLOSE(unit=2)
          CLOSE(unit=1)
        ENDIF !cli$present('P1')

        CALL exit(1)

  898   CALL lib$put_output('Trimcard:Failed to open output-file: '//file2(1:lout))
        CALL exit(iostat_to_status(iostat))
  899   CALL lib$put_output('trimcard:Failed to open input-file: '//file(1:lout))
        CALL exit(iostat_to_status(iostat))

 1000   FORMAT(a)
 2000   FORMAT(a<lout>)
        END

! word wrapping, not perfect, but better than nothing :-)
        integer function delimit(string)
        character*(*) string

        delimit=LEN(string)
        do while (delimit.gt.1)
         if (INDEX(' .,;:?!]})>-+=',string(delimit:delimit)).GT.0) return
         delimit=delimit-1
        end do
        end
