/* <TITLE>UNIX_TO_VMS Translates a Unix file  spec to an OpenVMS file spec </TITLE> */
/* 
   original vms_to_unix:
   19-JUN-2000, David Mathog, Biology Division, Caltech
   unix_to_vms:
   2002-02-28 J.Huber to do the inverse unix-to-vms

   Translates a Unix file spec/path to an OpenVMS file spec .  
   These are written to stdout.  In addition, if it translates to
   a file name, that is stored in the symbol VMS_FILENAME.
   Otherwise the symbol is left empty (for no translations).
   (If multiple names are converted, VMS_FILENAME contains the last one.)

   09-AUG-2000, David Mathog, Biology Division, Caltech
   If the first parameter is "" then it reads from stdin and writes
   to stdout.

   The translation policies are up to CRTL routine DECC$to_vms,
   it depends on the C RTL library version, and eventually from DECC features 
   like 
  DEFINE DECC$DISABLE_TO_VMS_LOGNAME_TRANSLATION "ENABLE"
  translates foo/bar to [.foo]bar, otherwise foo:[bar].

*/
#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include <descrip.h>
#include <unixlib.h>
#include <unixio.h>
#include <string.h>
#include <lib$routines.h>

#define MAXFILENAME 4096
static char symbol_value[MAXFILENAME];  /* should be more than long enough */
static struct  dsc$descriptor_s  symbol_value_desc;

static char vfilename[MAXFILENAME];
static char *res;

int emit(char *string, int dummy){
   strcpy(symbol_value,string); 
   fprintf(stdout,"%s\n",string);
   return(1);
}
int main(int argc, char *argv[]){
int temp;

  $DESCRIPTOR(symbol_name_desc,"VMS_FILENAME");

  symbol_value_desc.dsc$b_class =  DSC$K_CLASS_S;
  symbol_value_desc.dsc$b_dtype =  DSC$K_DTYPE_T;

  symbol_value[0]='\0';
  switch(argc){
     case 2:
        if(*argv[1] == '\0'){
           res=vfilename;  /* initialize it to something which isn't NULL */
           while (res != NULL){
              res=fgets(vfilename,MAXFILENAME,stdin);
              if(res != NULL){
                temp=strlen(vfilename);
                vfilename[temp-1]='\0'; /* obliterate new line */
                (void) decc$to_vms(vfilename, emit, 1, 0);
              }
           }
        }
        else {
           (void) decc$to_vms(argv[1], emit, 1, 0);
        }
        break;
     default:
        (void) fprintf(stderr,
"\n usage: \n\n  unix_to_vms path \n\n");        
        (void) fprintf(stderr,
"  path is any valid Unix file or directory name (wildcards allowed )\n");        
        (void) fprintf(stderr,
"    When vmsfilespec is \"\"  the program reads filenames from\n");
        (void) fprintf(stderr,"    and translates those.\n");
        (void) fprintf(stderr,
"    Only names for extant directories and/or files will be translated\n");
  }
  symbol_value_desc.dsc$w_length = strlen(symbol_value);
  symbol_value_desc.dsc$a_pointer =  symbol_value;
  (void) LIB$SET_SYMBOL(&symbol_name_desc,&symbol_value_desc);
}
