*	Language: VAX Fortran
*	Author: J.Huber, MPI Muenchen
*	Creation date: April'88
*	Revision date: 7-July-88
*+
*	MTUTIL.FOR
* 	=============================================
* 
* 
*       	MT$UTIL package:
*  <title>     	general tape I/O routines (logical block I/O) </title>
*        
* 
*         All routines may be called as Logical or Integer functions:
*         the function result is the condition code of QIO's done inside
*         the routines, if it resulted in a system service error, or
*         (in normal cases) the result of the I/O function(s),i.e.
*         usually the function result is the first longword of IOSB.
*         Since the high 16 bit of IOSB(1) may contain transfer counts,
*         the code should not be compared to SS$_NORMAL, but rather asked
*         for logical .TRUE. (=success) or .FALSE. (=QIO or I/O error).
*         Another way would be to equivalence the return code with an integer*2
*         value, then the result compares to SS$_normal.
*
*         All QIO use logical I/O functions, i.e.,
*         LOG_IO privilege is required, or (the normal case),
*         the tape drive in use must be MOUNTed FOReign and ALLOCated !
*
*         In all calls, the arguments CHAN and IOSB have the same meaning:
*         CHAN = VMS I/O channel assigned: word value returned by the
*                initial MT$ASSIGN call, to be used in calls to other routines.
*         IOSB = I/O status block (2 longwords)
*                (for details see the I/O User's Ref Manual)
*
*       Routines descriptions:
*
*	---------------------------------------------------------------------
*       ......	MT$ASSIGN ... assign tape ,set/get density , position
*        
*       	COND= MT$ASSIGN (DEVICE,CHAN,DEN,IOSB ,POSITION)
*        
*	  DEVICE = VMS deviname string of tape drive (e.g. 'MSA0:')
*         DEN  = density  (800,1600,6250,833) where applicable
*                0 = do not change current tape density 
*       	 (tape characteristic is returned in IOSB(2))
*         833 is valid for TK50/TK70 only !
*         MS: type drives do not select density by software, so always use 0
*             and select density via front panel switches.
*
*        POSITION = string to define tape position (optional)
*       	'R'	Rewind tape (position to -BOT-)
*       	'A'	Append: position to end of last file on tape
*       	'N'	New file: position behind last file on tape
*       	'S'	Stay - the tape position does not change
*        
* 
*	---------------------------------------------------------------------
*       ......	MT$APPEND ... position at end of last file for append
*        
*       	COND= MT$APPEND (CHAN,IOSB)
*
*	---------------------------------------------------------------------
* 
*       ......	MT$OPEN ... position at end of volume to write new file
*        
*       	COND= MT$OPEN (CHAN,IOSB)
*
*	---------------------------------------------------------------------
*	......  MT$BLOCK_IO  ... read/write tape block
*
*          COND = MT$BLOCK_IO (CHAN,LENGTH, BLOCK, IOSB,FUN,SWAP)
*        
*       	LENGTH = length to be written/buffer size for read
*       	BLOCK =  block buffer of at least <BLOCKLENGTH> bytes
*       	FUN =    -1 for read   +1 for write
*       	SWAP=1  ----> swap bytes (for conversion IBM->DEC)
*        
*        
*	   MT$BLOCK_IO is intended for package internal use;
*	   for usage convenience, there are the two entries
*	   for block read and write:
*
*          COND = MT$READ  (CHAN,LENGTH, BLOCK, IOSB)        read block
*          COND = MT$REVERSE  (CHAN,LENGTH, BLOCK, IOSB)     read block reverse
*
*          COND = MT$WRITE (CHAN,LENGTH, BLOCK, IOSB)        write block
*        
*	---------------------------------------------------------------------
*       .....	MT$CLOSE.... close output tape (write --EOV--)
*        
*       	COND = MT$CLOSE (CHAN,IOSB)
*        
*       	writes -EOV- to tape (2 tapemarks),
*       	positions tape before the last tape mark for next file write
* 
* 
*	---------------------------------------------------------------------
*        
*       ....	MT$SPF	.... skip file .............
*        
*          COND= MT$SPF ( CHAN , NFILES , IOSB)
*        
*	---------------------------------------------------------------------
*       ....	MT$SPB	.... skip physical block .........
*        
*          COND= MT$SPB (CHAN , NBLOCK, IOSB)
* 
*	---------------------------------------------------------------------
* 
*        
*       .....	MT$REWIND  .... rewind tape .............
*        
*       	COND= MT$REWIND(CHAN,IOSB,wait)
* 
* 
*	---------------------------------------------------------------------
*        
*       	MT$SENSE	...... sense characteristic .........
*        
*       	COND= MT$SENSE(CHAN,IOSB)
*        
*    	Tape characteristic and position is returned in second status word.
* 
*	---------------------------------------------------------------------
*
*        Example:
*         Logical MT$ASSIGN,MT$WRITE,MT$CLOSE,MT$UNLOAD
*         Integer IOSB(2),Chan
*         If (.not.MT$assign('MSA0:',Chan,0,Iosb,'R')) Then ...!error
*         ...
*         If (.not.MT$write(Chan,2048,data,Iosb)) ... !write data,error ?
*         ... loop for more records ...
*         If (.not.MT$close(chan,Iosb)) ...   !write EOV, check error
*         If (.not.MT$unload(chan,Iosb)) ...  !rewind and unload
*        
*	---------------------------------------------------------------------
*        
*       	MT$DENSITY	...... set density .........
*        
*       	COND= MT$DENSITY(CHAN,IOSB,DEN)
*		Integer DEN = density (800,1600,6250,833)
*	---------------------------------------------------------------------
*-

	Logical Function MT$ASSIGN (DEVICE,CHAN,DEN,IOSB,POSITION)
	Character*(*) Device
	Character Position
	Integer*2 Chan
	Integer*4 Den,Iosb(2)

	Logical  SYS$ASSIGN,SYS$QIOW
	Logical  MT$SENSE,MT$SPF,MT$SPB,MT$REWIND,MT$DENSITY
	Logical Cond
	Integer*2 IOstate
 	Equivalence (COND,IOstate)

	Include '($SSDEF)'
	Include '($IODEF)'
	Include '($MTDEF)'

	Cond = SYS$ASSIGN (Device, CHAN,,)
	If (Cond) Then 
	 If (Position.ne.'S') Cond = MT$REWIND(chan,iosb,.TRUE.)
!	 Cond= MT$DENSITY(CHAN,IOSB,DEN) !sense characteristics,set density
	 If (Position.eq.'A') call MT$APPEND(chan,iosb)
	 If (Position.eq.'N') call MT$OPEN  (chan,iosb)
	 Cond = MT$SENSE (Chan,Iosb)	 
	Endif

900	Continue
	MT$assign = Cond
	Return	
	End

	Logical Function MT$DENSITY(CHAN,IOSB,DEN)
	Integer*2 Chan,Den
	Integer*4 Iosb(2)

	Logical  MT$SENSE,MT$SETMODE,MT$REWIND
	Logical Cond
	Integer*2 IOstate
 	Equivalence (COND,IOstate)

	Include '($SSDEF)'
	Include '($IODEF)'
	Include '($MTDEF)'
	Include 'MTSENSE.INC'
	 Cond= MT$SENSE(CHAN,IOSB)	!sense characteristics
	 If (Cond) Then
	  If (Den.ne.0) Then
*	   clear density bits in sensed mode
	   Sensed_Mode = Sensed_mode .and..not.(MT$K_GCR_6250.or.
     *         MT$K_PE_1600.or.MT$K_NRZI_800.or.MT$K_BLK_833)
	   If (Den.eq.800) Sensed_Mode = Sensed_Mode.or.MT$K_NRZI_800
	   If (Den.eq.1600) Sensed_Mode = Sensed_Mode.or.MT$K_PE_1600
	   If (Den.eq.6250) Sensed_Mode = Sensed_Mode.or.MT$K_GCR_6250
	   If (Den.eq.833) Sensed_Mode = Sensed_Mode.or.MT$K_BLK_833
	   Cond = MT$REWIND(CHAN,IOSB,.TRUE.)
	   If (Cond) Cond= MT$SETMODE(CHAN,IOSB)	!set new characteristics
	  Endif
	 Endif
	MT$DENSITY = COND
	Return
	End


	Logical Function MT$SENSE (CHAN,IOSB)
	Integer*2 Chan
	Integer*4 Iosb(2)
*	Returns in IOSB(2) the tape characteristics

	Include '($IODEF)'
	Include '($MTDEF)'
	Include 'MTSENSE.INC'

*	MTSENSE.INC
*	Integer Sense_buffer(3)
*	Integer Sensed_mode
*	Equivalence (Sensed_mode,Sense_buffer(2))
	Logical MT$SETMODE 
	Logical  SYS$QIOW
	Integer IOFunction

	IOFunction = IO$_SENSEMODE
10	Continue
        MT$SENSE = SYS$QIOW (,%VAL(CHAN),%VAL(IOFunction),IOSB,,,
     &    %REF(Sense_buffer),%VAL(12),,,,)
	If (MT$SENSE) MT$SENSE = IOSB(1)
	Return

	Entry MT$SETMODE (CHAN,IOSB)
	IOFunction = IO$_SETMODE
	Go to 10
	End

	Logical Function MT$SPF (CHAN,Nfiles,IOSB)
	Integer*2 Chan
	Integer*4 Nfiles,Iosb(2)
*	Skip over Nfiles files (Tapemarks)
	Logical MT$SPB

	Include '($IODEF)'
	Include '($MTDEF)'

	Logical  SYS$QIOW
	Integer IOFunction

	IOFunction = IO$_SKIPFILE
10	Continue
        MT$SPF = SYS$QIOW (,%VAL(CHAN),%VAL(IOFunction),IOSB,,,
     &    %VAL(Nfiles),,,,,)
	If (MT$SPF) MT$SPF = IOSB(1)
	Return

*	Skip over Nblocks Records (Blocks)
	Entry MT$SPB (CHAN,Nfiles,IOSB)
	IOFunction = IO$_SKIPRECORD
	Go to 10
	End


	Logical Function MT$BLOCK_IO (CHAN,LENB,BLOCK,IOSB,FUN,SWAP)
	Integer*2 Chan
	Integer*4 Iosb(2)
	Integer Fun,Swap,LENB
	Byte Block

*	Read/Write a logical block on tape

	Include '($IODEF)'
	Include '($MTDEF)'

	Logical  SYS$QIOW,Doswap
	Integer IOFunction,IOfunction_C

	Logical MT$READ,MT$REVERSE,MT$WRITE

	IOFunction = FUN
	DoSwap = (swap.gt.0) 

1	Continue
	If (IOFunction.gt.0) Then
	 IOfunction_C = IO$_WRITELBLK
	 If (doswap) Call MT$swap (LENB,block)
	ElseIf (IOfunction.eq.-2) Then
	 IOfunction_C = IO$_READLBLK.OR.IO$M_REVERSE	!Read reverse
	Else
	 IOfunction_C = IO$_READLBLK
	Endif
	
10	Continue
        MT$BLOCK_IO = SYS$QIOW (,%VAL(CHAN),%VAL(IOfunction_C),IOSB,,,
     &    %REF(block),%VAL(LENB),,,,)
	
	If (MT$BLOCK_IO) Then
	 MT$BLOCK_IO = IOSB(1)
	 If ((IOFunction.lt.0).and.DoSwap) Then
	  Call MT$Swap (LENB,block)
	 Endif
	Endif
	Return

	Entry MT$READ  (CHAN,LENB, BLOCK, IOSB)
	IOFunction = -1
20	DoSwap = .false.
	Go to 1

	Entry MT$REVERSE  (CHAN,LENB, BLOCK, IOSB)
	IOFunction = -2
	Go to 20
*
	Entry MT$WRITE (CHAN,LENB, BLOCK, IOSB)
	IOFunction = 1
	Go to 20
	End


	Subroutine MT$swap(len,block)
*	swap bytes subroutine
	integer len,i,n
	integer*2 block(*),itemp
	byte btemp(2)
	Equivalence (itemp,btemp(1))

	n = len/2
	Do i=1,n
	 itemp = block(i)
	 btemp(1) = btemp(2)
	 btemp(2) = block(i)
	 block(i) = itemp
	Enddo
	Return
	End
	 

	Logical Function MT$REWIND (CHAN,IOSB,Wait)
	Integer*2 Chan
	Integer*4 Iosb(2)
	logical Wait
*	Rewind tape, wait until rewound if Wait .true.
	Logical MT$UNLOAD 

	Include '($IODEF)'
	Include '($MTDEF)'

	Logical  SYS$QIOW
	Integer IOFunction

	IOFunction = IO$_REWIND
10	Continue
	If (.not.Wait) IOFunction = IOFunction.or.IO$M_NOWAIT
        MT$REWIND = SYS$QIOW (,%VAL(CHAN),%VAL(IOFunction),IOSB,,,,,,,,)
	If (MT$REWIND) MT$REWIND = IOSB(1)
	Return

 	Entry MT$UNLOAD (CHAN,IOSB,Wait)
	IOFunction = IO$_UNLOAD
	Go to 10
	End

	Logical Function MT$CLOSE (CHAN,IOSB)
*	write EOV on tape, position before the 2nd of the tapemarks

	Integer*2 Chan
	Integer*4 Iosb(2)

	Include '($SSDEF)'
	Include '($IODEF)'
	Include '($MTDEF)'

	Logical  SYS$QIOW
	Integer*2 MT$SPB,IC

        MT$close = SYS$QIOW (,%VAL(CHAN),%VAL(IO$_WRITEOF),IOSB,,,,,,,,)
        MT$close = SYS$QIOW (,%VAL(CHAN),%VAL(IO$_WRITEOF),IOSB,,,,,,,,)
	If (MT$CLOSE) MT$CLOSE = IOSB(1)
        IC = MT$SPB (chan,-1,iosb)	!position before 2'nd tapemark
*	   the normal return code from space over tapemark must be EOF !
*	   anything else is an error
	If (IC.ne.SS$_ENDOFFILE) MT$close = IC
	Return
	End

	Logical Function MT$APPEND (CHAN,IOSB)
	Integer*2 Chan
	Integer*4 Iosb(2)
	Character Position

	Logical  MT$SPF,MT$SPB
	External  MT$SPF,MT$SPB
	Logical MT$OPEN
	Logical Cond
	Integer*2 IOstate
 	Equivalence (Cond,IOstate)

	Include '($SSDEF)'
	Include '($IODEF)'
	Include '($MTDEF)'

	Position = 'A'
100	Continue
	IOstate = SS$_NORMAL
	Do While ((IOstate.eq.SS$_NORMAL).or.(IOstate.eq.SS$_ENDOFFILE))
	 Cond = MT$SPF (Chan,32676,Iosb)
	End Do
	If (Position.eq.'A'.and.IOstate.eq.SS$_ENDOFVOLUME)
     &	    Cond = MT$SPB (Chan,-1,Iosb)
	MT$append = Cond
	Return

	Entry MT$OPEN(chan,iosb)
	Position = 'N'
	Go to 100	!join common code
	End
