      INTEGER FUNCTION SPAWN_AND_READ(cmd_str,answer_buf,nb)
C
C-----------------------------------------
C
C   Author   :- J.Huber                9-MAY-1995
C
C=========================================
C
c      INTEGER FUNCTION SPAWN_AND_READ(cmd_str,answer_buf,nb)
c
c
c   Purpose   :<title> spawn a DCL command and return the output of the command </title>
c   Inputs    : character*(*) cmd_str: DCL command to execute
c               method: by string descriptor
c               byte(*) answer_buf:    returned sys$output of command
c               method: by reference, address of a byte array
c                       (.e.g. array in Fortran, array of char in C)
c               nb: number of bytes maximum (dimension of answer_buf)
c               method: by reference, address of a short integer
c   Outputs   : answer_buf filled with text written to sys$output
c               lines are separated by <CR><LF> (Ascii code 13,10)
c               nb         number of characters returned in answer_buf
c   Function result: VMS status code
c                    test the low order bit for success (Fortran: TRUE)
c                    the code is either a VMS error code from one of the
c                    system calls inside the routine, or (the normal
c                    case) the final return status of the spawned
c                    command.
C
C=========================================
C +
C Declarations.
C -
      IMPLICIT NONE
      CHARACTER*(*) cmd_str
      BYTE answer_buf(*)
      INTEGER*2 nb

      INCLUDE '($LIBDEF)'
      INCLUDE '($SSDEF)'
      INCLUDE '($IODEF)'
      INCLUDE '($SYSSRVNAM)'
      INCLUDE '($CLIDEF)'
      INCLUDE '($DVIDEF)'
      INTEGER LIB$SPAWN,LIB$GET_EF
      EXTERNAL LIB$SPAWN,LIB$GET_EF

      INTEGER CR,LF
      PARAMETER (CR=13)
      PARAMETER (LF=10)
      INTEGER EF,EFIO,efmask,st,cstatus,sflags,iofunc
      INTEGER*2 iosb(4)
      INTEGER*2 chan
      LOGICAL done
      INTEGER bufptr
      INTEGER remlen

      STRUCTURE /itmlst/
        UNION
          MAP
            INTEGER*2 buflen,itmcod
            INTEGER*4 bufadr,retadr
          END MAP
          MAP
            INTEGER*4 end_list
          END MAP
        END UNION
      END STRUCTURE

      RECORD /itmlst/ dvilst(2)
      CHARACTER*64 mbxdev
C + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
C Entry Point.
C - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

      efmask=0
      spawn_and_read = LIB$GET_EF(EF)
      IF (.NOT. spawn_and_read) RETURN
      efmask = ibset(efmask,MOD(ef,32))
      spawn_and_read = LIB$GET_EF(EFIO)
      IF (.NOT. spawn_and_read) GOTO 903
      efmask = ibset(efmask,MOD(efio,32))

*        create and assign mailbox for sys$output

      spawn_and_read = SYS$CREMBX( ,chan ,255 ,4096 , , ,)
      IF (.NOT. spawn_and_read) GOTO 902

      dvilst(1).itmcod = DVI$_DEVNAM
      dvilst(1).buflen = LEN(mbxdev)
      dvilst(1).bufadr = %LOC(mbxdev)
      dvilst(1).retadr = 0
      dvilst(2).end_list = 0
      spawn_and_read = SYS$GETDVI(,%VAL(chan),,dvilst,iosb,,,,)
      IF (.NOT. spawn_and_read) GOTO 901

*     spawn the DCL command
      sflags = CLI$M_NOWAIT.OR.CLI$M_NOCLISYM.OR.CLI$M_NOLOGNAM.OR.
     &  CLI$M_NOKEYPAD
      spawn_and_read = LIB$SPAWN (cmd_str,'NL:',mbxdev,sflags,,,cstatus,
     &  ef)
      IF (.NOT. spawn_and_read) GOTO 901

      done = .FALSE.
      bufptr = 1
      remlen = nb
      nb = 0
      iofunc = IO$_READVBLK

      DO WHILE (.NOT.done)
        st = SYS$QIO(%VAL(EFIO),%VAL(chan),%VAL(iofunc),iosb,,,
     &           %ref(answer_buf(bufptr)),%val(MIN(255,remlen)),,,,)

*     wait for subprocess or I/O completion
        CALL SYS$WFLOR(%VAL(EF),%VAL(efmask))
        st = SYS$CLREF(%VAL(EF))
        IF (st .EQ. SS$_WASSET) THEN
          done = .true.
          IF (iosb(1).EQ.0) CALL sys$cancel(%VAL(chan))
        ENDIF
        IF (st) st = iosb(1)
        IF (st) THEN
          bufptr = bufptr + iosb(2)
          answer_buf(bufptr)=CR !insert <CR>
          answer_buf(bufptr+1)=LF !insert <LF>
          bufptr = bufptr+2
          nb = nb + iosb(2)+2
          remlen = remlen - iosb(2) -2
        ENDIF
      ENDDO

*     flush mailbox until empty or buffer exhausted
      iofunc = IO$_READVBLK.OR.IO$M_NOW
      st = 1
      DO WHILE (st.AND.(remlen.GT.0))
        st = SYS$QIOW(%VAL(EFIO),%VAL(chan),%VAL(iofunc),iosb,,,
     &           %ref(answer_buf(bufptr)),%val(MIN(255,remlen)),,,,)
        IF (st) st = iosb(1)
        IF (st) THEN
          bufptr = bufptr + iosb(2)
          answer_buf(bufptr)=CR !insert <CR>
          answer_buf(bufptr+1)=LF !insert <LF>
          bufptr = bufptr+2
          nb = nb + iosb(2)+2
          remlen = remlen - iosb(2) -2
        ENDIF
      ENDDO

  901 CALL SYS$DASSGN ( chan )
  902 CALL lib$free_ef(EFIO)
  903 CALL lib$free_ef(EF)
*      return subprocess exit status if no previous error
      IF (spawn_and_read) spawn_and_read = cstatus
      END
