/* 
<title>  C stream I/O package </title>
        STREAM_F.C;
C-----------------------------------------
C
C   Author   :- J.Huber                4-MAR-1991
C
C=========================================

   The stream package has been written, because I needed a way to read files
   written on Unix files systems.
   
   I decided NOT to return the C file-pointer in the argument-list, because e.g.
   fortran-interoperability on 64-bit system like DecUnix would have problem
   using integers. Therefore a fixed number of internal file-pointers
   is used, and the index number returned to the calling program.

   Stream_finit is needed for systems which do not zero static global variables.
   On VMS the entry stream_finit_vms() also calls DECC$CRTL_INIT, so the package
   can be used from other languages than C.
*/
 

#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#ifdef __VMS
#include <descrip.h>
#include <signal.h>
#endif 

#include "stream_f.h"

#define STREAM_MXFID 4   
FILE *stream_fp[STREAM_MXFID];
#define MAX16B 0x10000
#define MIN(x,y) ((x) < (y) ? (x) : (y))

/* stream_finit   initialize C I/O system      */

void stream_finit(void) {

int i;
for (i=0; i < STREAM_MXFID; i++) stream_fp[i] = 0;
}

#ifdef __VMS
void stream_finit_vms(void) {
  DECC$CRTL_INIT();
  stream_finit();
}
#endif
/*

      INTEGER FUNCTION STREAM_FCLOSE(fid)
C=========================================
C   Purpose   :
C   stream IO package for Fortran programs: CLOSE files
C   Interface to the C runtime routine fclose
C
C   Returned value  : status = 0 if no error
C   Inputs    :
C    int fid       by reference  : file ID  (as returned by stream_fopen)
C   Outputs   :
C    fid reset to 0
C
C fid has nothing in common with VMS FID (File ID), it's pure internal.
C=========================================

*/

int stream_fclose(int *fid)
{
  int ret;
  FILE *ft;

  ft = stream_fp[(*fid)-1];
  errno = 0;
  ret = fclose(ft);
  if (errno != 0) perror("fclose error");
  *fid = 0;
  return ret;
}
/*

      INTEGER FUNCTION STREAM_FOPEN(fid,FILE_NAME,MODE)
C=========================================
C   Purpose   :
C   stream IO package for Fortran programs: OPEN files
C   Interface to the C run-time routine fopen
C
C   Returned value  : status >0, fid>0 : success, status=fid=0: open error
C   Inputs    :
C    char file_name by descriptor : VMS file-specification
C    char mode      by descriptor : string to indicate access mode (R/W..)
C   Outputs   :
C    int fid       by reference  : file-id returned
C
C    The output argument fid should be saved over all calls to this package
C    (stream_fread,_fwrite,_fclose). It serves as a context variable to
C    identify the open stream, so Several streams may be open and used in the
C    same program unit.
C
C    For details of the MODE string, see the description of fopen in
C    the VAX-C runtime library manual.
C    Shortly, it consists of up to 3 letters:
C    1. r/w/a   for read/write/append
C    2. +       update access (should be avoided for this package)
C    3. b       binary : no conversion of carriage control (recommended)
C=========================================

*/

#ifdef __VMS
int stream_fopen(int *fid,struct dsc$descriptor_s *file_name,
		   struct dsc$descriptor_s *mode)
{
#else
int stream_fopen(int *fid,char *file_name,char *mode)
{
#endif
  char *temp_fname;
  char *temp_mode;
  int index;

  *fid = 0;
  for (index=0; index < STREAM_MXFID; index++) if (stream_fp[index] == 0) break;
  if (index >= STREAM_MXFID) return 0;
  if (stream_fp[index] != 0) return 0;

#ifdef __VMS
  temp_fname = malloc(256);
  temp_mode = malloc(256);
  strncpy(temp_fname,file_name->dsc$a_pointer,file_name->dsc$w_length);
  strncpy(temp_mode,mode->dsc$a_pointer,mode->dsc$w_length);
  /*       insert terminating \0 into the string arguments*/
  temp_fname[file_name->dsc$w_length] = 0;
  temp_mode[mode->dsc$w_length] = 0;
#else
  temp_fname = file_name;
  temp_mode = mode;
#endif
  /*       open the file*/
#ifdef __VMS
  stream_fp[index] = fopen(temp_fname,temp_mode,"mbc=32","mbf=32");
  free(temp_fname);
  free(temp_mode);
#else
errno = 0;
  stream_fp[index]  = fopen(temp_fname,temp_mode);
if ((errno != 0)|| (stream_fp[index] == 0)) {
	perror("stream_fopen error");
	return 0;
}	
#endif
  *fid= (index+1);
  return (index+1);
}
/*

      INTEGER FUNCTION STREAM_FREAD(fid,buffer,length)
C=========================================
C   Purpose   :
C   stream IO package for Fortran programs: read from file
C   Interface to the C runtime routine fopen
C
C   Returned value  : number of bytes read, error if 0
C   Inputs    :
C    int fid       by reference  : fileID (as returned by stream_FOPEN)
C    buffer         by reference  : address of buffer to receive data
C    length         by reference  : number of bytes to read
C   Outputs   :
C
C=========================================
*/

int stream_fread(int *fid,void *buffer,int *length)
{

  int ret,itemlen,remlen,donelen;
  char *buf;
  FILE *ft;

  ft = stream_fp[(*fid)-1];
  donelen = 0;
  remlen = *length;
  buf = buffer;
  ret = 0;
  /*       read items of maximum MAX16B from  the file*/

  while (remlen > 0)
  {
    if (ferror(ft) || feof(ft)) return 0;
    itemlen = MIN(remlen,MAX16B);
    ret = fread(buf,1,itemlen,ft);
    if (ferror(ft) || feof(ft)) return 0;
    if (ret > 0)
    {
      donelen = donelen + ret;
      remlen = remlen - MAX16B;
      buf += MAX16B;
    }
    else
    {
      remlen = 0;
    }
  }
  if (ret > 0) ret = donelen;
  return ret;
}
/*

      INTEGER FUNCTION STREAM_FWRITE(fid,buffer,length)
C=========================================
C   Purpose   :
C   stream IO package for Fortran programs: write to file
C   Interface to the C runtime routine fopen
C
C   Returned value  : number of bytes written, error if <=0
C   Inputs    :
C    int fid       by reference  : file ID  (as returned by stream_FOPEN)
C    buffer         by reference  : address of buffer to write
C    length         by reference  : length of buffer (bytes)
C   Outputs   :
C
C=========================================

*/

int stream_fwrite(int *fid,void *buffer,int *length)
{

  int itemlen,remlen,donelen,ret;
  char *buf;
  FILE *ft;

  ft = stream_fp[(*fid)-1];
  donelen = 0;
  remlen = *length;
  buf = buffer;
  ret = 0;
  /*       write items of maximum MAX16B to the file*/

  while (remlen > 0)
  {
    itemlen = MIN(remlen,MAX16B);
    ret = fwrite(buf,1,itemlen,ft);
    if (ret > 0)
    {
      donelen = donelen + ret;
      remlen = remlen - MAX16B;
      buf += MAX16B;
    }
    else
    {
      remlen = 0;
    }
  }
  if (ret > 0) ret = donelen;
  return ret;
}

/* 
     INTEGER STREAM_FSEEK(fid,offset,direction)

   stream_fseek: set file position 

   offset: integer bytes to move position
   direction:from were to start positiong

   See help cc run-time fseek for the direction values
   Default for direction=0 is SEEKSET, i.e. offset is from begin of file
   The current definition in DECC is
      #define SEEK_SET 0
      #define SEEK_CUR 1
      #define SEEK_END 2
*/

 int stream_fseek(int *fid, int *offset, int *direction)
{
 
 FILE *ft;
 int dir;

  ft = stream_fp[(*fid)-1];
  dir = *direction;
  if (dir == 0) dir = SEEK_SET; /* default offset is from begin of file */
  return fseek( ft, *offset, dir);
}
